# frozen_string_literal: true

require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')
  include Rack::Test::Methods

  MUTATE_PARAMS = {
    dalo: 'Boolean',
    entry_date: 'DateTime',
    external_ref: 'String',
    communicate_exact_address: 'Boolean',
    typology: 'String',
    lessor_status: 'String',
    type: 'String',
    age: 'String',
    district: 'String',
    department: 'String',
    publication_end_date: 'DateTime',
    rent_amount: 'Float',
    surface: 'Integer',
    rooms: 'Integer',
    exclusivity_end_date: 'DateTime',
    nb_of_eligible_employees: 'Integer',
    nb_of_requests_from_employees: 'Integer',
    availability_date: 'DateTime',
    description: 'String',
    details: 'String',
    # effort_rate: 'Float',
    min_effort_treshold: 'Float',
    reserved: 'Boolean',
    priority_public: 'Boolean',
    mobility: 'Boolean',
    bedrooms: 'Integer',
    bathrooms: 'Integer',
    heating_type: 'String',
    hot_water_distribution: 'String',
    floor: 'Integer',
    floors: 'Integer',
    elevators: 'Integer',
    has_gardian: 'Boolean',
    outside: 'String',
    cellars: 'Integer',
    parking_charges: 'Float',
    parking_type: 'String',
    reserving_siren: 'String',
    address_complements: 'String',
    pmr: 'Boolean',
    kitchen_type: 'String',
    address: 'String',
    postal_code: 'String',
    district: 'String',
    country: 'String',
    latitude: 'Float',
    longitude: 'Float',
    mutation: 'Boolean',
    threshold_type: 'String',
    threshold_subtype: 'String',
    individual_heating: 'Boolean',
    date_publication_start: 'DateTime',
    date_publication_end: 'DateTime',
    kind: 'String',
    building_type: 'String',
    already_rent: 'Boolean',
    guarantee_deposit: 'Float',
    rental_charges: 'Float',
    offer_status: 'String',
    min_effort_rate: 'Float',
    tense_area: 'Boolean',
    subsidiary_company: 'String',
    rpls_id: 'String',
    operation_name: 'String',
    administrator: 'String',
    rent_with_charges: 'Float',
    reserving_business_name: 'String',
    administration_area: 'String',
    potential_request_name_town: 'String',
    potential_request_name_county: 'String',
    number_of_companies_30_minutes_around: 'Integer',
    construction_type: 'String',
    address_can_be_shared: 'String',
    qpv: 'Boolean',
    total_floors: 'String',
    address_extended: 'String',
    test_created_at: 'DateTime',
    already_inhabited: 'Boolean',
    new_offer: 'Boolean',
    als_siren: 'String',
    postal_code: 'String',
    insee_code: 'String',
    residence_code: 'String',
    residence_title: 'String',
    agency_id: 'String',
    gardens: 'Integer',
    green_area: 'Boolean',
    condo: 'Boolean',
    furnished_lodging: 'Boolean',
    virtual_visit: 'Boolean',
    duplex: 'Boolean',
    first_quartile: 'String',
    other_quartiles: 'String',
    lessor_convention_reference: 'String',
    reserving_agency_name: 'String',
    reserving_agency_siren: 'String',
    disability_accesses: 'Integer',
    terraces_or_balconies: 'Integer',
    mad_als_date: 'DateTime',
    reservation_agreement_ref: 'String',
    designation_date_end: 'DateTime',
    lessor_orientation: 'String',
    dpe_ges: 'String',
    dpe_conso: 'String',
    entities_siret: 'Array',
    commercialized: 'Boolean',
    is_public: 'Boolean',
    agency_name: 'String',
    lessor_contact_name: 'String',
    lessor_email: 'String',
    lessor_phone_number: 'String',
    single_parent: 'Boolean',
    auto_destroy_next_day: 'Boolean'
  }.freeze

  ACCEPTABLE_PARAMS = %w[
    dalo
    als_id
    external_ref
    communicate_exact_address
    typology
    type
    age
    district
    department
    publication_end_date
    rent_amount
    rent_with_charges
    surface
    rooms
    exclusivity_end_date
    availability_date
    description
    details
    effort_rate
    designation_date_end
    min_effort_treshold
    single_parent
    reserved
    already_rent
    reservation_agreement_ref
    priority_public
    mobility
    bedrooms
    bathrooms
    heating_type
    hot_water_distribution
    floor
    floors
    elevators
    has_elevator
    has_gardian
    outside
    cellars
    has_cellar
    parking_type
    pmr
    kitchen_type
    address
    postal_code
    country
    latitude
    longitude
    address_complements
    mutation
    individual_heating
    date_publication_start
    date_publication_end
    kind
    building_type
    guarantee_deposit
    rental_charges
    parking_charges
    offer_status
    offer_status_id
    offer_status_updated_at
    lessor_status
    lessor_status_id
    lessor_status_updated_at
    min_revenues
    total_charges
    min_effort_rate
    tense_area
    subsidiary_company
    rpls_id
    operation_name
    administrator
    reserving_business_name
    reserving_siren
    administration_area
    potential_request_name_town
    potential_request_name_county
    number_of_companies_30_minutes_around
    construction_type
    construction_year
    address_can_be_shared
    qpv
    total_floors
    address_extended
    test_created_at
    already_inhabited
    new_offer
    lessor_name
    lessor_siren
    lessor_orientation
    als_siren
    first_quartile
    postal_code
    insee_code
    residence_code
    residence_title
    pictures
    pictures_nb
    agency_id
    agency_name
    lessor_contact_name
    lessor_email
    lessor_phone_number
    gardens
    has_garden
    green_area
    terraces_or_balconies
    dpe_ges
    dpe_conso
    condo
    furnished_lodging
    virtual_visit
    duplex
    first_quartile
    other_quartiles
    lessor_convention_reference
    reserving_agency_name
    reserving_agency_siren
    disability_accesses
    has_disability_access
    effort_rate
    lessor_id
    lessor
    entities_siret
    entity_ids
    agency_id
    reference
    release_date
    commercialized
    mad_als_date
    is_mad_updated_at
    auto_destroy_next_day
  ].freeze

  HOUSING_REQUESTS_ACCEPTABLE_PARAMS = %w[
    is_active
    request_status
    attributed_housing_district
    attributed_housing_district_postal_code
    old_request_status
    request_status_updated_at
    old_entity_status
    updated_entity_status
    entity_status
    dalo
    asked_mobility
    valid_mobility
    decider
    submitted_to_cal_nb
    sent_to_cal_nb
    rejected_in_cal_nb
    rejected_after_cal_nb
    accepted_in_cal_nb
    mobility
    seeked_types
    entity_help_required
    contract_type
    registration_number
    applicant_ids
    entity_validated
    priority_points_indexes
    nud
    nb_of_requests_made
    reference_tax_revenues
    seeked_districts
    category_type
    single_parent
    id_counter
    id_unique
    id_unique_associate
    internal_ref
    forcing_duplicate
    electronical_renewal
    ec_law_ressources
    is_long_delay_anormal
    date_calculator_indicator
    with_car_park
    widen_request
    housing_type_list
    expulsion_judgement_date
    other_reason
    work_contract_type
    seeked_district_postal_codes
    seeked_departments
    seeked_localisation_list
    dependents
    monthly_revenues
    typology_range
    entity_validated
    family_size
    coapplicants
    lessor_name
    lessor_siren
    social_housing_entry_date
    social_housing_name
    test_created_at
    sne_number
    employee_id
    employee_start_date
    entity_access_level
    employee_unit
    tab_state
    motivation1
    motivation2
    motivation3
    seeking_first_floor
    seeking_elevator
    seeked_max_rent
    sne_creation_date
    precisions
    has_multiple_attributions_under_conditions
    seeked_housing_categories
    min_surface
    main_applicant_id
    requester_firstname
    requester_lastname
    requester_fullname
    requester_email
    entity_id
    coapplicant_ids
    coapplicants
    seeked_epci_first_quartils
    seeked_quartil_belongings
    dependents_monthly_incomes
    dependents_monthly_charges
    housing_offer_ids
    radiation_date
    radiation_motive

  ].freeze

  VALID_OPERATORS = %w[
    $in
    $nin
    $ne
    $gt
    $gte
    $lt
    $lte
    $eq
    $regex
    $all
  ].freeze

  HOUSING_APPLICATIONS_ACCEPTABLE_PARAMS = %w[
    housing_request_id
    housing_offer_id
    epci
    max_date
    applicated_at
    effort_rate
    status_updated_at
  ].freeze

  HOUSING_APPLICATIONS_MUTATE_PARAMS = {
    epci: 'Integer',
    max_date: 'Date',
    status: 'String'
  }.freeze

  describe Api::Dmo::HousingOffersController, type: :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:responsible) {create(:user, roles: ['Administrator'])}
    let!(:user) {create(:user)}

    describe 'POST /housing_offers' do
      describe 'Create housing_offer with minimal valid datas' do
        before :each do
          authenticated_header(request, responsible)
          post 'create',
               data: {
                 attributes: {}
               }
        end
        it 'returns status code 201' do
          expect(response).to have_http_status(201)
        end
      end
      describe 'Create housing_offer on existing one' do
        let!(:lessor) {create(:lessor, user_id: user.id, title: 'Lessor')}
        let!(:housing_offer) {create(:housing_offer, user_id: user.id,  lessor_id: lessor.id.to_s,auto_destroy_next_day:true,
                                     external_ref: "111" , typology:"T1")}
        before :each do
          authenticated_header(request, user)
          post 'create',
               data: {
                 attributes: {
                   lessor_id: lessor.id.to_s,
                   external_ref: "111" ,
                   auto_destroy_next_day:true,
                   typology:"T2"
                 }
               }
          perform_enqueued_jobs do
            enqueued_jobs.each { |job| job[:job].perform_now(*job[:args]) }
          end

        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
          expect(json['data']['id']).to eq(housing_offer.id.to_s)
          expect(json['data']['attributes']["typology"]).to eq("T2")
        end
      end
      describe 'Create housing_offer autodelete' do
        let!(:lessor) {create(:lessor, user_id: user.id, title: 'Lessor')}

        let!(:housing_offer) {create(:housing_offer, user_id: user.id,  lessor_id: lessor.id.to_s,auto_destroy_next_day:true,
                                     external_ref: "111" , typology:"T1")}
        before :each do
          authenticated_header(request, user)
          post 'create',
               data: {
                 attributes: {
                   lessor_id: lessor.id.to_s,
                   external_ref: "111" ,
                   auto_destroy_next_day:true,
                   typology:"T2"
                 }
               }
          housing_offer.set(updated_at: Time.now, last_update_by_lessor: Time.now - 37.hours)
          perform_enqueued_jobs do
            enqueued_jobs.each { |job| job[:job].perform_now(*job[:args]) }
          end

        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
          expect(housing_offer.reload.offer_status).to eq("RETIRE PAR LE BAILLEUR")
          expect(json['data']['id']).to eq(housing_offer.id.to_s)
          expect(json['data']['attributes']["typology"]).to eq("T2")
        end
      end
      MUTATE_PARAMS.each do |param, type|
        describe 'post request with ' + param.to_s + ' (' + type.to_s + ') is valid' do
          let(:params) do
            attr = {}
            attr[param] = get_typed_value(type)
            attr
          end
          before :each do
            authenticated_header(request, responsible)
            post 'create',
                 data: {
                   attributes: params
                 }
          end
          it 'returns status code 201 and the value appears in json' do
            expect(response).to have_http_status(201)
            if type == 'Date' || type == 'DateTime'
              expect(json['data']['attributes'][param.to_s]).not_to be_empty
              expect(json['data']['attributes'][param.to_s]).not_to eq('')
            else
              expect(json['data']['attributes'][param.to_s]).to eq(get_typed_value(type))
            end
          end
        end
      end
      describe 'post request with construction_year is valid' do
        before :each do
          authenticated_header(request, responsible)
          post 'create',
               data: {
                 attributes: {construction_year: '2001'}
               }
        end
        it 'returns status code 201 and the value appears in json' do
          expect(response).to have_http_status(201)
          expect(Date.parse(json['data']['attributes']['construction_year'])).to eq(Date.new(2001, 0o1, 0o1))
        end
      end
      describe 'post request with lessor_id is valid' do
        let!(:lessor) {create(:lessor, user_id: responsible.id, title: 'Lessor')}
        before :each do
          authenticated_header(request, responsible)
          post 'create',
               data: {
                 attributes: {lessor_id: lessor.id.to_s}
               }
        end
        it 'returns status code 201 and the value appears in json' do
          expect(response).to have_http_status(201)
          expect(json['data']['attributes']['lessor_id']).to eq(lessor.id.to_s)
        end
      end
    end

    describe 'GET /housing_offers' do
      describe 'GET /housing_offers' do
        let!(:housing_offer1) {create(:housing_offer, terraces_or_balconies: 50)}
        let!(:housing_offer3) {create(:housing_offer, terraces_or_balconies: 150)}
        let!(:housing_offer2) {create(:housing_offer, terraces_or_balconies: 150)}
        before :each do
          authenticated_header(request, responsible)
          get 'index'
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(3)
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(3)
        end
      end

      describe 'GET /housing_offers/id/all_applications' do
        let!(:housing_offer1) { create(:housing_offer, terraces_or_balconies: 50) }
        let!(:user) {create(:user)}
        let!(:applicant) {create(:applicant)}
        let!(:housing_request1) {create(:housing_request, motivation1: 'acceuillant', documents_required: 50, main_applicant_id: applicant.id.to_s)}
        let!(:housing_request2) {create(:housing_request, motivation1: 'acceuillant', documents_required: 50, main_applicant_id: applicant.id.to_s)}
        let!(:housing_request3) {create(:housing_request, motivation1: 'acceuillant', documents_required: 50, main_applicant_id: applicant.id.to_s)}
        let!(:housing_application1) {create(:housing_application, housing_offer_id: housing_offer1.id.to_s,
                                            housing_request_id: housing_request1.id.to_s)}
        let!(:housing_application2) {create(:housing_application, housing_offer_id: housing_offer1.id.to_s,
                                            housing_request_id: housing_request2.id.to_s)}
        let!(:housing_application3) {create(:housing_application, housing_offer_id: housing_offer1.id.to_s,
                                            housing_request_id: housing_request3.id.to_s)}
        before :each do
          authenticated_header(request, responsible)
          get 'all_applications', {id: housing_offer1.id}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          ap json
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(3)
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(3)
        end
      end

      ACCEPTABLE_PARAMS.each do |acceptable_param|
        type = MUTATE_PARAMS[acceptable_param.to_sym] || MUTATE_PARAMS[acceptable_param.to_s]
        next unless type

        describe 'GET /housing_offers with sort on field ' + acceptable_param + ' works ' do
          let!(:housing_offer3) {create(:housing_offer, acceptable_param => get_typed_value(type, 2))}
          let!(:housing_offer1) {create(:housing_offer, acceptable_param => get_typed_value(type, 0))}
          let!(:housing_offer2) {create(:housing_offer, acceptable_param => get_typed_value(type, 1))}
          before :each do
            authenticated_header(request, responsible)
            get 'index', sort: {acceptable_param => 1}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            if type == 'DateTime'
              housing_offer_attribute = json['data'].map {|unit| DateTime.parse(unit['attributes'][acceptable_param])}
              expect(housing_offer_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
            elsif type == 'Date'
              housing_offer_attribute = json['data'].map {|unit| Date.parse(unit['attributes'][acceptable_param])}
              expect(housing_offer_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
            else
              housing_offer_attribute = json['data'].map {|unit| unit['attributes'][acceptable_param]}
              expect(housing_offer_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
            end
          end
        end
      end

      describe 'GET /housing_offers with operators' do
        let!(:housing_offer1) {create(:housing_offer, description: 'acceuillant', terraces_or_balconies: 50)}
        let!(:housing_offer3) {create(:housing_offer, description: 'Super', terraces_or_balconies: 150)}
        let!(:housing_offer2) {create(:housing_offer, description: 'bel appart', terraces_or_balconies: 150)}
        describe 'with $lt' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', terraces_or_balconies: {'$lt': 100}
          end
          it 'returns results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(1)
          end
        end
        describe 'with $in' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', description: {'$in': %w[acceuillant flamand]}
          end
          it 'returns results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(1)
          end
        end
        describe 'with $ne' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', terraces_or_balconies: {'$ne': 50}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(2)
          end
        end
        describe 'with $regex' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', description: {'$regex': /^S/}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(1)
          end
        end
      end

      describe 'GET /housing_offers search by postal codes' do
        let!(:offers_1) { create_list :housing_offer, 2, postal_code: '93240' }
        let!(:offers_rand) { create_list :housing_offer, 4, postal_code: '2'.ljust(5, rand(9999).to_s) }
        let!(:offers_2) { create_list :housing_offer, 2, postal_code: '75100' }

        before do
          authenticated_header(request, responsible)
          get 'index', postal_codes: ['75100', '93240']
        end

        it 'succeeds' do
          expect(response.status).to eq(200)
          expect(json['meta']['pagination']['total_objects']).to eq 8
        end

        it 'orders results' do
          expect(json['data'][0]['attributes']['postal_code']).to eq '75100'
          expect(json['data'][2]['attributes']['postal_code']).to eq '93240'
        end
      end
    end

    describe 'GET /housing_offers/:id' do
      describe 'verif it work' do
        let!(:housing_offer1) {create(:housing_offer, description: 'acceuillant', terraces_or_balconies: 50)}
        before :each do
          authenticated_header(request, responsible)
          get 'show', id: housing_offer1.id
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
        end
      end

      describe 'verif it work not work with a non authorized user' do
        let!(:housing_offer1) {create(:housing_offer, description: 'acceuillant', terraces_or_balconies: 50)}

        before :each do
          authenticated_header(request, user)
          get 'show', id: housing_offer1.id
        end
        it 'returns status code 404' do
          expect(response).to have_http_status(404)
        end
      end

      context 'with housing applications' do
        let!(:threshold1) do
          create(
            :threshold,
            type: 'PLA LM',
            thresholds_map: {
              'Paris et communes limitrophes': {
                'thresholds_list': [
                  13_050,
                  21_272,
                  27_883,
                  30_521,
                  36_307,
                  40_859
                ],
                'another_person': 4552
              },
              'IDF hors Paris et Communes limitrophes': {
                'thresholds_list': [
                  13_050,
                  21_272,
                  25_569,
                  28_075,
                  33_238,
                  37_401
                ],
                'another_person': 4166
              },
              'Autres régions': {
                'thresholds_list': [
                  11_342,
                  16_525,
                  19_872,
                  22_111,
                  25_870,
                  29_155
                ],
                'another_person': 3252
              }
            }
          )
        end

        let!(:applicant1) do
          create(
            :applicant,
            amount_year_minus_two: 10_346.0,
            monthly_incomes: {
              'SAL': 864.32
            }
          )
        end
        let!(:request1) do
          create(
            :housing_request,
            main_applicant: applicant1,
            is_active: true,
            seeked_departments: %w[12]
          )
        end

        let!(:applicant2) do
          create(
            :applicant,
            amount_year_minus_two: 10_346.0,
            monthly_incomes: {
              'SAL': 864.32
            }
          )
        end
        let!(:request2) do
          create(
            :housing_request,
            main_applicant: applicant2,
            is_active: true,
            seeked_departments: %w[12]
          )
        end

        let!(:applicant3) do
          create(
            :applicant,
            amount_year_minus_two: 10_346.0,
            monthly_incomes: {
              'SAL': 864.32
            }
          )
        end
        let!(:request3) do
          create(
            :housing_request,
            main_applicant: applicant3,
            is_active: true,
            seeked_departments: %w[12]
          )
        end

        let!(:offer1) do
          create(
            :housing_offer,
            department: '12',
            typology: 'T2',
            threshold_type: 'PLA LM',
            threshold_subtype: 'Autres régions',
            rent_amount: 324.12
          )
        end
        let!(:offer2) do
          create(
            :housing_offer,
            department: '12',
            typology: 'T2',
            threshold_type: 'PLA LM',
            threshold_subtype: 'Autres régions',
            rent_amount: 324.12
          )
        end

        let!(:application1_1) do
          create(
            :housing_application,
            housing_request: request1,
            housing_offer: offer1,
            status: 'TRANSMIS BAILLEUR'
          )
        end
        let!(:application1_2) do
          create(
            :housing_application,
            housing_request: request1,
            housing_offer: offer2,
            status: 'TRANSMIS BAILLEUR'
          )
        end

        let!(:application2_1) do
          create(
            :housing_application,
            housing_request: request2,
            housing_offer: offer1,
            status: 'TRANSMIS BAILLEUR'
          )
        end
        let!(:application2_2) do
          create(
            :housing_application,
            housing_request: request2,
            housing_offer: offer2,
            status: 'TRANSMIS BAILLEUR'
          )
        end

        let!(:application3_1) do
          create(
            :housing_application,
            housing_request: request3,
            housing_offer: offer1,
            status: 'TRANSMIS BAILLEUR'
          )
        end

        describe 'GET /with_validated_applications without from' do
          before :each do
            authenticated_header(request, responsible)
            get 'with_validated_applications'
          end

          it 'returns error' do
            expect(response).to_not have_http_status(200)
          end
        end

        describe 'GET /with_validated_applications' do
          before :each do
            authenticated_header(request, responsible)
            get 'with_validated_applications', from: (Time.now - 1.minute).to_s
          end

          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end

          it 'returns a valid json' do
            expect(json).not_to be_empty
            expect(json['data'].length).to be 2
          end
        end

        describe 'GET /with_validated_applications without pagination' do
          before :each do
            authenticated_header(request, responsible)
            get 'with_validated_applications', from: (Time.now - 1.minute).to_s, pagination: 'false', per_page: '1'
          end

          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end

          it 'returns a valid json' do
            expect(json).not_to be_empty
            expect(json['data'].length).to be 2
          end
        end

        describe 'GET /with_validated_applications/offer1.id' do
          before :each do
            authenticated_header(request, responsible)
            get 'with_validated_applications_for', id: offer1.id.to_s, from: (Time.now - 1.hours).to_s
          end

          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end

          it 'returns a valid json' do
            expect(json).not_to be_empty
            expect(json['data'].length).to be 3
          end
        end

        describe 'GET /with_validated_applications/offer1.id without pagination' do
          before :each do
            authenticated_header(request, responsible)
            get 'with_validated_applications_for', id: offer1.id.to_s, from: (Time.now - 1.hours).to_s, pagination: 'false', per_page: '1'
          end

          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end

          it 'returns a valid json' do
            expect(json).not_to be_empty
            expect(json['data'].length).to be 3
          end
        end

        describe 'GET /with_validated_applications/offer2.id' do
          before :each do
            authenticated_header(request, responsible)
            get 'with_validated_applications_for', id: offer2.id.to_s, from: (Time.now - 1.hours).to_s
          end

          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end

          it 'returns a valid json' do
            expect(json).not_to be_empty
            expect(json['data'].length).to be 2
          end
        end
      end
    end

    describe 'PATCH /housing_offers/:id' do
      housing_offer_params = {}
      MUTATE_PARAMS.each do |param, type|
        housing_offer_params[param] = get_typed_value(type)
      end

      MUTATE_PARAMS.each do |param, type|
        describe 'patch request with ' + param.to_s + ' ' + type.to_s + ' is valid' do
          let!(:housing_offer) {create(:housing_offer, param.to_sym => get_typed_value(type))}
          let(:params) do
            attr = {}
            attr[param] = get_typed_value(type, 2)
            attr
          end
          before :each do
            authenticated_header(request, responsible)
            patch 'update',
                  id: housing_offer.id.to_s,
                  data: {
                    attributes: params
                  }
          end
          it 'returns status code 200 and the value appears in json' do
            expect(response).to have_http_status(200)
            if type == 'Date' || type == 'DateTime' || type == 'Array[Coord]'
              expect(json['data']['attributes'][param.to_s]).not_to be_empty
              expect(json['data']['attributes'][param.to_s]).not_to eq('')
            else
              expect(json['data']['attributes'][param.to_s]).to eq(get_typed_value(type, 2))
            end
          end
        end
      end
    end

    describe 'patch request with change in ordered_picture_ids is valid' do
      let!(:housing_offer1) {create(:housing_offer, user_id: responsible.id)}
      let!(:picture0) {create(:picture, housing_offer_id: housing_offer1.id.to_s, user_id: responsible.id)}
      let!(:picture1) {create(:picture, housing_offer_id: housing_offer1.id.to_s, user_id: responsible.id)}
      let!(:picture2) {create(:picture, housing_offer_id: housing_offer1.id.to_s, user_id: responsible.id)}
      let!(:picture3) {create(:picture, housing_offer_id: housing_offer1.id.to_s, user_id: responsible.id)}
      let!(:picture4) {create(:picture, housing_offer_id: housing_offer1.id.to_s, user_id: responsible.id)}

      before :each do
        authenticated_header(request, responsible)
        patch 'update',
              id: housing_offer1.id.to_s,
              data: {
                attributes: {
                  ordered_picture_ids: [picture4.id.to_s, picture3.id.to_s, picture2.id.to_s, picture1.id.to_s, picture0.id.to_s]
                }
              }
      end
      it 'returns status code 200 and the value appears in json' do
        expect(response).to have_http_status(200)
        expect(json['data']['attributes']['ordered_picture_ids']).to eq([picture4.id.to_s, picture3.id.to_s, picture2.id.to_s, picture1.id.to_s, picture0.id.to_s])
      end
    end

    describe 'post selected application' do
      let!(:housing_offer1) {create(:housing_offer)}

      let!(:user7) {create(:user)}
      let!(:applicant7) {create(:applicant, user: user7)}
      let!(:housing_request7) {create(:housing_request, main_applicant_id: applicant7.id.to_s, als_id: 'als_id7')}
      let!(:housing_applications7) do
        create(:housing_application,
               housing_offer_id: housing_offer1.id.to_s,
               housing_request_id: housing_request7.id.to_s,
               status: 'PLO')
      end
      let!(:user8) {create(:user)}
      let!(:applicant8) {create(:applicant, user: user8)}
      let!(:housing_request8) {create(:housing_request, main_applicant_id: applicant8.id.to_s, als_id: 'als_id8')}
      let!(:housing_applications8) do
        create(:housing_application,
               housing_offer_id: housing_offer1.id.to_s,
               housing_request_id: housing_request8.id.to_s,
               status: 'PLO')
      end

      before :each do
        authenticated_header(request, responsible)
        post 'post_selected_applications', id: housing_offer1.id, data: {attributes: {als_request_ids: [housing_request7.als_id, housing_request8.als_id]}}
      end
      it 'return valid json' do
        expect(json).not_to be_empty
        expect(response).to have_http_status(200)
        data = json['data']
        expect(data.length).to eq(2)
      end
    end

    describe 'DELETE /housing_offers/:id' do
      let!(:housing_offer) {create(:housing_offer, user_id: responsible.id.to_s)}
      before :each do
        authenticated_header(request, responsible)

        delete 'destroy',
               id: housing_offer.id.to_s
      end
      it 'returns status code 404' do
        expect(response).to have_http_status(204)
      end
    end

    describe 'GET /housing_offers/:id/detailed_score' do
      let(:demandeur) {create :user, roles: ['Demandeur']}
      let(:group) {create :group, user_ids: [user.id, demandeur.id]}
      let(:appl_1) {create :applicant}
      let(:appl_2) {create :applicant, main_user_id: user.id}
      let(:appl_3) {create :applicant, main_user_id: demandeur.id}
      let!(:score_1) {create :score, housing_offer_id: housing_offer.id, applicant_id: appl_1.id}
      let!(:score_2) {create :score, housing_offer_id: housing_offer.id, applicant_id: appl_2.id}
      let!(:score_3) {create :score, housing_offer_id: housing_offer.id, applicant_id: appl_3.id}

      let!(:housing_offer) {create(:housing_offer, readers: ["g_#{group.id}"])}

      context 'as admin' do
        before do
          authenticated_header(request, responsible)
        end

        it 'requires applicant_id params' do
          get 'detailed_score', {id: housing_offer.id}
          expect(response).to have_http_status 406
          expect(json['errors'][0]['message']).to eq 'param is missing or the value is empty: applicant_id'
        end

        it "succeeds" do
          get 'detailed_score', {id: housing_offer.id, applicant_id: appl_1.id}
          expect(response).to have_http_status 200
          expect(json['data'].length).to eq 1
          expect(json['data'][0]['id']).to eq score_1.id.to_s
        end
      end

      context 'as housing_offer reader' do
        before do
          authenticated_header(request, user)
        end

        it 'requires applicant_id params' do
          get 'detailed_score', {id: housing_offer.id}
          expect(response).to have_http_status 406
          expect(json['errors'][0]['message']).to eq 'param is missing or the value is empty: applicant_id'
        end

        it 'returns error when not applicant reader' do
          get 'detailed_score', {id: housing_offer.id, applicant_id: appl_1.id}
          expect(response).to have_http_status 403
        end

        it 'succeeds' do
          get 'detailed_score', {id: housing_offer.id, applicant_id: appl_2.id}
          expect(response).to have_http_status 200
          expect(json['data'].length).to eq 1
          expect(json['data'][0]['id']).to eq score_2.id.to_s
        end
      end

      context 'as demandeur' do
        before do
          authenticated_header(request, demandeur)
          get 'detailed_score', {id: housing_offer.id}
        end

        it "succeeds" do
          expect(response).to have_http_status 200
          expect(json['data'].length).to eq 1
          expect(json['data'][0]['id']).to eq score_3.id.to_s
        end
      end
    end

    #Charlie's tests
    context 'housing_offers with ' do
      let!(:housing_offer1) do
        create(:housing_offer, is_mad: true, user: responsible, readers: [user.id.to_s])
      end
      let!(:housing_offer2) {create(:housing_offer, user: responsible)}

      describe 'GET /housing_offers with is_mad' do
        before :each do
          authenticated_header(request, responsible)
          get 'index', is_mad: true
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns a valid json' do
          expect(json).not_to be_empty
          data = json['data']
          expect(data.length).to eq(1)
          expect(data.first['id']).to eq(housing_offer1.id.to_s)
          expect(data.first['attributes']['read_level']).to eq('write')
          expect(data.first['attributes']['readers']).to include(user.id.to_s)
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(1)
        end
      end

      describe 'PATCH /housing_offers with new writers' do
        let!(:group1) {create(:group, user_ids: [user.id])}
        before :each do
          authenticated_header(request, responsible)
          patch 'update', id: housing_offer2.id, data: {attributes: {writers: ["g_#{group1.id}"]}}
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns a valid json' do
          expect(json).not_to be_empty
          data = json['data']
          expect(data['id']).to eq(housing_offer2.id.to_s)
          expect(data['attributes']['read_level']).to eq('write')
          expect(data['attributes']['writers']).to include("g_#{group1.id}")
        end
      end
    end

    context 'with basic data' do
      let!(:applicant) {create(:applicant, user: user)}
      let!(:housing_request) do
        create(
          :housing_request,
          is_active: true,
          main_applicant: applicant,
          user: user
        )
      end

      let!(:lessor) {create(:lessor)}

      let!(:housing_offer1) do
        create(
          :housing_offer,
          user: responsible,
          lessor: lessor
        )
      end
      let!(:housing_offer2) do
        create(
          :housing_offer,
          user: responsible,
          lessor: lessor
        )
      end
      let!(:housing_offer3) do
        create(
          :housing_offer,
          user: responsible,
          lessor: lessor
        )
      end

      describe 'GET /housing_offers' do
        before :each do
          authenticated_header(request, responsible)
          get 'index'
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(3)
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(3)
        end
      end

      describe 'GET /housing_offers/:id' do
        context 'as responsible' do
          before :each do
            authenticated_header(request, responsible)
            get 'show', id: housing_offer1.id
          end

          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end

          it 'returns valid JSON' do
            subject
            expect(json['data']['id']).to eq(housing_offer1.id.to_s)
            expect(json['data']['attributes']['lessor']['title']).to eq(lessor.title)
            expect(json['data']['attributes'].key?('reference')).to be_truthy
            expect(json['data']['attributes'].key?('release_date')).to be_truthy
          end
        end

        context 'with inaccessible housing_offer' do
          before :each do
            authenticated_header(request, responsible)
            get 'show', id: 'test_unavailable_housing_offer'
          end

          it 'returns status code 404' do
            expect(response).to have_http_status(404)
          end
        end
      end

      describe 'PUT /housing_offers/:id' do
        describe 'update my housing_offer' do
          let(:valid_params) do
            {
              id: housing_offer2.id.to_s,
              data: {attributes: {rent_amount: 245.62}}
            }
          end

          before :each do
            authenticated_header(request, responsible)
            put 'update', valid_params
          end

          it 'returns status code 200' do
            expect(response.status).to eq(200)
          end

          it 'updated form' do
            expect(json['data']['id']).to eq(housing_offer2.id.to_s)
            expect(json['data']['attributes']['rent_amount']).to eq(245.62)
          end
        end
      end
    end

    context 'with scored data' do
      let!(:admin) {create(:administrator)}

      let!(:threshold1) do
        create(
          :threshold,
          type: 'PLA LM',
          thresholds_map: {
            'Paris et communes limitrophes': {
              'thresholds_list': [
                13_050,
                21_272,
                27_883,
                30_521,
                36_307,
                40_859
              ],
              'another_person': 4552
            },
            'IDF hors Paris et Communes limitrophes': {
              'thresholds_list': [
                13_050,
                21_272,
                25_569,
                28_075,
                33_238,
                37_401
              ],
              'another_person': 4166
            },
            'Autres régions': {
              'thresholds_list': [
                11_342,
                16_525,
                19_872,
                22_111,
                25_870,
                29_155
              ],
              'another_person': 3252
            }
          }
        )
      end

      let!(:user1) {create(:user)}
      let!(:applicant1) do
        create(
          :applicant,
          sne_profile_creation_date: '2016-09-24',
          main_user_id: user1.id,
          amount_year_minus_two: 9787.24,
          monthly_incomes: {
            'Salaire': 1045.23,
            'APL': 321.62
          }
        )
      end
      let!(:request1) do
        create(
          :housing_request,
          is_active: true,
          main_applicant: applicant1,
          seeked_districts: ['34507'],
          seeked_departments: ['34']
        )
      end

      let!(:user2) {create(:user, auth_roles: ['Bailleur'])}
      let!(:applicant2) do
        create(
          :applicant,
          sne_profile_creation_date: '2017-09-24',
          main_user_id: user2.id,
          amount_year_minus_two: 9787.24,
          monthly_incomes: {
            'Salaire': 1045.23,
            'APL': 321.62
          }
        )
      end
      let!(:request2) do
        create(
          :housing_request,
          is_active: true,
          main_applicant: applicant2,
          seeked_districts: ['34507'],
          seeked_departments: ['34']
        )
      end

      let(:group1) {create(:group, user_ids: [user2.id])}

      let!(:offer1) do
        create(
          :housing_offer,
          rent_amount: 234.32,
          rental_charges: 42.15,
          parking_charges: 12.71,
          typology: 'T2',
          offer_status: 'LOGEMENT PUBLIE',
          surface: 29,
          threshold_type: 'PLA LM',
          department: '34',
          min_effort_rate: 0,
          threshold_subtype: 'IDF hors Paris et Communes limitrophes',
          readers: ["g_#{group1.id}"]
        )
      end

      let!(:offer2) do
        create(
          :housing_offer,
          rent_amount: 434.32,
          rental_charges: 53.12,
          parking_charges: 22.43,
          typology: 'T2',
          offer_status: 'LOGEMENT PUBLIE',
          surface: 34,
          threshold_type: 'PLA LM',
          department: '34',
          min_effort_rate: 0,
          threshold_subtype: 'Autres régions',
          readers: ["g_#{group1.id}"]
        )
      end

      let!(:offer3) do
        create(
          :housing_offer,
          rent_amount: 434.32,
          rental_charges: 53.12,
          parking_charges: 22.43,
          typology: 'T2',
          surface: 34,
          threshold_type: 'PLA LM',
          department: '34',
          min_effort_rate: 0,
          threshold_subtype: 'Autres régions',
          readers: ["g_#{group1.id}"]
        )
      end

      describe 'GET as user1' do
        before :each do
          expect(user1.reload.main_applicant_id).to eq applicant1.id
          expect(applicant1.reload.eligibility_query).to_not eq '{is_public:true}'
          drain_all_jobs
          authenticated_header(request, user1)
          get 'index'
        end

        it 'returns 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid json' do
          data = json['data']
          expect(data.length).to be 2
        end
      end

      describe 'GET /id as user1' do
        before :each do
          drain_all_jobs
          authenticated_header(request, user1)
          get 'show', id: offer1.id.to_s
        end

        it 'returns 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid json' do
          data = json['data']
          expect(data['id']).to eq offer1.id.to_s
        end
      end

      describe 'handles queued jobs' do
        it 'should have jobs and right quantity of scores after perform' do
          expect(enqueued_jobs).to be_present
          expect(enqueued_jobs.size).to be > 0
          drain_all_jobs

          scores = Hermes::Score.where(housing_offer_id: offer1.id)
          expect(scores.count).to eq(2)
        end
      end

      describe 'as user1 with sort on profile_epci_score' do
        before :each do
          drain_all_jobs
          authenticated_header(request, user2)
          get 'index'
        end

        it 'returns 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid json' do
          data = json['data']
          expect(data.length).to be 3
        end
      end

      describe 'as user1 with sort on profile_epci_score' do
        before :each do
          drain_all_jobs
          authenticated_header(request, user1)
          get 'index'
        end

        it 'returns 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid json' do
          data = json['data']
          expect(Hermes::HousingOffer.where(offer_status: 'LOGEMENT PUBLIE').count).to be 2
          expect(Hermes::HousingOffer.where(offer_status: 'LOGEMENT PUBLIE').count).to be 2
          expect(data.length).to be 2
        end
      end

      describe 'as user1 with sort on profile_epci_score' do
        before :each do
          drain_all_jobs
          authenticated_header(request, user1)
          get 'index', sort: {'profile_epci_score' => -1}
        end

        it 'returns 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid json' do
          data = json['data']
          expect(
            data.map {|elt| elt['attributes']['profile_epci_score']}.each_cons(2).all? {|a, b| b <= a}
          ).to be true
          expect(data.length).to be 2
        end
      end

      describe 'as user1 with profile_epci_score > 0' do
        before :each do
          drain_all_jobs
          authenticated_header(request, user1)
          get 'index', profile_epci_score: {'$gt' => '0'}
        end

        it 'returns 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid json' do
          data = json['data']
          expect(
            data.map {|elt| elt['attributes']['profile_epci_score']}
          ).to all(be > 0)
          expect(data.length).to be 1
        end
      end

      describe 'as user1 with sort on profile_epci_score >= 0' do
        before :each do
          drain_all_jobs
          authenticated_header(request, user1)
          get 'index', profile_epci_score: {'$gte' => '0'}
        end

        it 'returns 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid json' do
          data = json['data']
          expect(
            data.map {|elt| elt['attributes']['profile_epci_score']}
          ).to all(be >= 0)
          expect(data.length).to be 2
        end
      end

      describe 'as admin check offer1 side effects' do
        before :each do
          drain_all_jobs
          authenticated_header(request, admin)
          get 'show', id: offer1.id
        end

        it 'returns code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid JSON' do
          attributes = json['data']['attributes']
          expect(attributes['eligible_requests_nb']).to eq(2)
          expect(attributes['applicated_nb']).to eq(0)
        end
      end

      describe 'check the visibilty of offer depending on the offer status and user type' do
        before :each do
          perform_enqueued_jobs do
            enqueued_jobs.each do |job|
              if job[:job] == 'UpdateScoreJob'
                UpdateScoreJob.perform_now(*job[:args])
              end
            end
          end
        end

        it 'returns code 200' do
          authenticated_header(request, user1)
          get 'index'
          expect(response).to have_http_status(200)
        end

        it 'as admin' do
          authenticated_header(request, admin)
          get 'index'
          data = json['data']
          expect(data.length).to be 3

          patch 'update',
                id: offer2.id.to_s,
                data: {
                  attributes: {
                    offer_status: 'RETIRE PAR LE BAILLEUR'
                  }
                }

          get 'index'
          data = json['data']
          expect(data.length).to be 3
        end

        it 'as bailleur' do
          authenticated_header(request, user2)
          get 'index'
          data = json['data']
          expect(data.length).to be 3

          patch 'update',
                id: offer2.id.to_s,
                data: {
                  attributes: {
                    offer_status: 'RETIRE PAR LE BAILLEUR'
                  }
                }

          get 'index'
          data = json['data']
          expect(data.length).to be 3
        end

        it 'as user' do
          authenticated_header(request, user1)
          get 'index'
          data = json['data']
          expect(data.length).to be 2

          authenticated_header(request, admin)
          patch 'update',
                id: offer2.id.to_s,
                data: {
                  attributes: {
                    offer_status: 'RETIRE PAR LE BAILLEUR'
                  }
                }

          authenticated_header(request, user1)
          get 'index'
          data = json['data']
          expect(data.length).to be 1
        end
      end

      # TODO: correct spec, since behaviour seems ok
      describe 'as admin with offer1 check eligible requests' do
        before :each do
          drain_all_jobs
          authenticated_header(request, admin)
          get 'eligible_requests', id: offer1.id
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid JSON' do
          data = json['data']
          expect(data.length).to eq(2)
          first_attributes = data[0]['attributes']
          expect(
            data.map do |line|
              [line['attributes']['eligible_position'], line['attributes']['profile_epci_score']]
            end
          ).to all(all(be_a Integer))
        end
      end

      describe 'as admin with offer1 check eligible requests sorting with score data' do
        before :each do
          drain_all_jobs
          authenticated_header(request, admin)
          get 'eligible_requests', id: offer1.id, sort: {'profile_epci_score' => -1}
        end

        it 'returns 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid json' do
          data = json['data']
          expect(
            data.map {|elt| elt['attributes']['profile_epci_score']}
              .each_cons(2)
              .all? {|a, b| b <= a}
          ).to be true
          expect(data.length).to be 2
        end
      end

      describe 'as admin with offer1 check eligible requests sorting with score data and filters' do
        before :each do
          drain_all_jobs
          authenticated_header(request, admin)
          get 'eligible_requests', id: offer1.id, sort: {'profile_epci_score' => -1}, main_applicant_id: applicant2.id.to_s
        end

        it 'returns 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid json' do
          data = json['data']
          expect(
            data.map {|elt| elt['attributes']['profile_epci_score']}
              .each_cons(2)
              .all? {|a, b| b <= a}
          ).to be true
          expect(data.length).to be 1
        end
      end

      context 'with acl and roles' do
        let!(:entity1) {create(:entity, siret: '123')}
        let!(:lessor1) {create(:lessor, siret: '456')}
        let!(:user_acl) {create(:user, auth_roles: ['Entreprise'])}

        let!(:offer4) {create(:housing_offer, entities_siret: ['123'], lessor_siren: '456')}

        describe 'GET for user_acl' do
          before :each do
            user_acl.groups.destroy
            group6 = Group.find_by(id: entity1.supervisor_group_id)
            group6.user_ids = [user_acl.id]
            group6.save
            authenticated_header(request, user_acl.reload)
            get 'index'
          end

          it 'returns 200' do
            expect(response).to have_http_status(200)
          end

          it 'returns valid json' do
            data = json['data']
            expect(data.length).to be 1
            offer = data.first
            attributes = offer['attributes']
            expect(offer['id']).to eq offer4.id.to_s
            expect(attributes['lessor_id']).to eq lessor1.id.to_s
            expect(attributes['readers']).to include(
                                               "g_#{entity1.reload.supervisor_group_id}"
                                             ).and(include("g_#{lessor1.reload.supervisor_group_id}"))
            expect(attributes['writers']).to be_nil
            expect(offer4.reload.writers).to include(
                                               "g_#{entity1.reload.manager_group_id}"
                                             ).and(include("g_#{lessor1.reload.manager_group_id}"))
          end
        end
      end
    end

    context 'as reservataire' do
      let!(:reservation_holder1) {create(:reservation_holder)}
      let!(:reservation_holder2) {create(:reservation_holder)}
      let!(:user1) {create(:user, auth_roles: ['Réservataire'], reservation_holder_id: reservation_holder1.id)}
      let!(:user2) {create(:user, auth_roles: ['Réservataire'], reservation_holder_id: reservation_holder2.id)}
      let!(:user3) {create(:user, auth_roles: ['Demandeur'])}
      let!(:housing_offer1) {create(:housing_offer, reservation_holder_id: reservation_holder1.id)}
      let!(:housing_offer2) {create(:housing_offer, reservation_holder_id: reservation_holder2.id)}
      let!(:housing_offer3) {create(:housing_offer)}
      let!(:housing_offer4) {create(:housing_offer)}

      describe 'GET /housing_offers' do
        describe 'access correct offers' do
          before :each do
            authenticated_header(request, user1)
            get 'index', {}
          end
          it 'returns results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(1)
          end
        end
      end

      describe 'GET /housing_offers/:id' do
        describe 'has access' do
          before :each do
            authenticated_header(request, user1)
            get 'show', id: housing_offer1.id
          end
          it 'returns results' do
            expect(response.status).to eq(200)
          end
        end
        describe 'is forbidden to access' do
          before :each do
            authenticated_header(request, user1)
            get 'show', id: housing_offer2.id
          end
          it 'returns results' do
            expect(response.status).to eq(404)
          end
        end
      end

      describe 'PATCH /housing_offers/:id' do
        describe 'has access' do
          before :each do
            authenticated_header(request, user1)
            patch 'update',
                  id: housing_offer1.id.to_s,
                  data: {
                    attributes: {
                      type: 'test'
                    }
                  }
          end
          it 'returns results' do
            expect(response.status).to eq(200)
            expect(housing_offer1.reload.type).to eq('test')
          end
        end
        describe 'is forbidden to access' do
          before :each do
            authenticated_header(request, user1)
            patch 'update',
                  id: housing_offer2.id.to_s,
                  data: {
                    attributes: {
                      type: 'Type'
                    }
                  }
          end
          it 'returns results' do
            expect(response.status).to eq(404)
          end
        end
      end

      describe 'DELETE /housing_offers/:id' do
        describe 'has access' do
          before :each do
            authenticated_header(request, user1)
            delete 'destroy',
                   id: housing_offer1.id.to_s
          end
          it 'returns results' do
            expect(response.status).to eq(204)
          end
        end
        describe 'is forbidden to access' do
          before :each do
            authenticated_header(request, user1)
            delete 'destroy',
                   id: housing_offer2.id.to_s
          end
          it 'returns results' do
            expect(response.status).to eq(404)
          end
        end
      end

      describe 'GET /housing_offers/:id/history' do
        before do
          create_list(:lessor_objective_history, 4, housing_offer_id: housing_offer1.id)
          create_list(:lessor_objective_history, 4, housing_offer_id: housing_offer2.id)
        end

        describe 'has access' do
          before do
            authenticated_header(request, user1)
            get 'history', id: housing_offer1.id
          end

          it 'returns results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq 4
          end
        end

        describe 'is forbidden to access' do
          before do
            offer_id = [housing_offer2.id, housing_offer3.id, housing_offer4.id].sample
            authenticated_header(request, user1)
            get 'history', id: offer_id
          end

          it 'returns error' do
            expect(response.status).to eq(404)
          end
        end
      end
    end

    context 'as bailleur' do
      let(:bailleur) {create :user, auth_roles: ['Bailleur']}
      let(:housing_offer1) {create(:housing_offer, user_id: bailleur.id)}
      let(:housing_offer2) {create(:housing_offer)}

      describe 'GET /housing_offers/:id/history' do
        before do
          create_list(:lessor_objective_history, 4, housing_offer_id: housing_offer1.id)
          create_list(:lessor_objective_history, 4, housing_offer_id: housing_offer2.id)
        end

        describe 'has access' do
          before do
            authenticated_header(request, bailleur)
            get 'history', id: housing_offer1.id
          end

          it 'returns results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq 4
          end
        end

        describe 'is forbidden to access' do
          before do
            authenticated_header(request, bailleur)
            get 'history', id: housing_offer2.id
          end

          it 'returns error' do
            expect(response.status).to eq(404)
          end
        end
      end
    end
  end

  require_relative '../../swagger_helper'

  describe Api::Dmo::HousingOffersController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    TAGS_API_DMO_HOUSING_OFFERS = 'API_DMO_HOUSING_OFFERS'
    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
    let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}
    let!(:applicant) {create(:applicant, user: responsible)}
    let!(:housing_request) {create(:housing_request, main_applicant_id: applicant.id.to_s)}

    let!(:threshold1) do
      create(
        :threshold,
        type: 'PLA LM',
        thresholds_map: {
          'Paris et communes limitrophes': {
            'thresholds_list': [
              13_050,
              21_272,
              27_883,
              30_521,
              36_307,
              40_859
            ],
            'another_person': 4552
          },
          'IDF hors Paris et Communes limitrophes': {
            'thresholds_list': [
              13_050,
              21_272,
              25_569,
              28_075,
              33_238,
              37_401
            ],
            'another_person': 4166
          },
          'Autres régions': {
            'thresholds_list': [
              11_342,
              16_525,
              19_872,
              22_111,
              25_870,
              29_155
            ],
            'another_person': 3252
          }
        }
      )
    end

    context 'as admin' do
      let!(:admin) {create(:administrator)}

      path '/api/dmo/housing_offers' do
        post 'Create housing_offer' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          json_params[:lessor_id] = {type: :string}
          json_params[:construction_year] = {type: :number}
          parameter(
            name: :params,
            in: :body,
            schema: jsonify_params(json_params),
            description: '<span>' \
              '<div class="opblock opblock-delete" style="padding: 4px; align-items: center; display: flex">' \
              '<span class="opblock-summary-method" style="margin-right: 4px">WARNING</span>' \
              '<span class="no-style">Setting an array of siret within `entities_siret`, will ' \
              ' change its relationships with the corresponding entities.</span></div></span>'
          )

          response '201', 'housing_offer created' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:params) do
              jsonapi_attributes(
                {}
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_offers'
            end
          end
        end
      end

      path '/api/dmo/housing_offers' do
        get 'Retrieves all housing_offers' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          parameter name: 'ids[]', in: :query, type: :array, items: {type: :string}, collectionFormat: :multi, required: false
          parameter name: 'postal_codes[]', in: :query, type: :array, items: {type: :string}, collectionFormat: :multi, required: false
          ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = :string
            if MUTATE_PARAMS[acceptable_param]
              type = nice_type(MUTATE_PARAMS[acceptable_param])
            end
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end
          # VALID_OPERATORS.each do |valid_operator|
          #   parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
          # end

          security [Bearer: {}]
          paginator

          response '200', 'housing_offers found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let!(:housing_offers) {create_list(:housing_offer, 5)}

            include_context 'with integration test'
            include_context 'valid jsonapi collection' do
              let(:expected_collection) {housing_offers}
            end
          end
        end
      end

      path '/api/dmo/housing_offers/{id}' do
        get 'Retrieves a single housing_offer by id' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          response '200', 'housing_offer found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:housing_offer) {create(:housing_offer)}
            let(:id) {housing_offer.id}
            include_context 'with integration test'
            include_context 'valid jsonapi ressource' do
              let(:expected_ressource) {housing_offer}
            end
          end
        end
      end

      path '/api/dmo/housing_offers/{id}/eligible_requests' do
        get 'Retrieves eligible_requests relative to a given housing_offer' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          parameter name: 'ids[]', in: :query, type: :array, items: {type: :string}, collectionFormat: :multi, required: false
          security [Bearer: {}]
          HOUSING_REQUESTS_ACCEPTABLE_PARAMS.each do |acceptable_param|
            parameter name: acceptable_param.to_s, in: :query, required: false
          end
          paginator
          response '200', 'eligible_requests found' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:housing_offer) {create(:housing_offer)}
            let(:id) {housing_offer.id}

            include_context 'not containings errors'
            run_test!
          end
        end
      end

      path '/api/dmo/housing_offers/with_validated_applications' do
        let!(:applicants) do
          create_list(
            :applicant,
            5,
            amount_year_minus_two: 10_345.22,
            monthly_incomes: {
              'SAL': 1200.0
            }
          )
        end

        let!(:housing_requests) do
          applicants.map do |applicant|
            create(
              :housing_request,
              main_applicant_id: applicant.id,
              is_active: true,
              seeked_departments: %w[26]
            )
          end
        end
        let!(:housing_offers) do
          create_list(
            :housing_offer,
            5,
            rent_amount: 234.28,
            typology: 'T2',
            department: '26',
            threshold_type: 'PLA LM',
            threshold_subtype: 'Autres régions'
          )
        end
        let!(:housing_applications) do
          count = 0
          housing_offers.map do |offer|
            housing_requests.map do |request|
              create(
                :housing_application,
                housing_request_id: request.id,
                housing_offer_id: offer.id,
                status: 'TRANSMIS BAILLEUR'
              )
              count += 1
            end
          end.flatten
        end

        get 'Retrieves all housing_offers that contains a validated applications from a chosen date date' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          parameter name: 'from', in: :query, type: :datetime, required: true, description: 'minimal `DateTime` of `status` set'
          parameter name: 'to', in: :query, type: :datetime, required: false, description: 'maximal `DateTime` of `status` set'
          parameter name: 'pagination', in: :query, type: :boolean, required: false, description: 'set to `false`, it deactivates the pagination (returns all objects)'
          ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = :string
            if MUTATE_PARAMS[acceptable_param]
              type = nice_type(MUTATE_PARAMS[acceptable_param])
            end
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end
          # VALID_OPERATORS.each do |valid_operator|
          #   parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
          # end

          security [Bearer: {}]
          paginator

          response '200', 'housing_offers found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:from) {(Time.now - 2.hours).to_s}
            let(:to) {(Time.now + 1.hours).to_s}
            let(:pagination) {'false'}

            include_context 'with integration test'
            include_context 'valid jsonapi collection without pagination' do
              let(:expected_collection) {housing_offers}
            end
          end
        end
      end

      path '/api/dmo/housing_offers/with_validated_applications/{id}' do
        let!(:applicants) do
          create_list(
            :applicant,
            5,
            amount_year_minus_two: 10_345.22,
            monthly_incomes: {
              'SAL': 1200.0
            }
          )
        end
        let!(:housing_requests) do
          applicants.map do |applicant|
            create(
              :housing_request,
              main_applicant_id: applicant.id,
              is_active: true,
              seeked_departments: %w[26]
            )
          end
        end
        let!(:housing_offer) do
          create(
            :housing_offer,
            rent_amount: 234.28,
            typology: 'T2',
            department: '26',
            threshold_type: 'PLA LM',
            threshold_subtype: 'Autres régions'
          )
        end
        let!(:housing_applications) do
          housing_requests.map do |request|
            create(
              :housing_application,
              housing_request_id: request.id,
              housing_offer_id: housing_offer.id,
              status: 'TRANSMIS BAILLEUR'
            )
          end.flatten
        end

        get 'For a housing_offer, list all housing_application that contains "TRANSMIS BAILLEUR" as their status - from a chosen date date' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          parameter name: 'id', in: :path, type: :string, required: true, description: 'targeted housing_offer id'
          parameter name: 'from', in: :query, type: :datetime, required: true, description: 'minimal `DateTime` of `status` set'
          parameter name: 'to', in: :query, type: :datetime, required: false, description: 'maximal `DateTime` of `status` set'
          parameter name: 'pagination', in: :query, type: :boolean, required: false, description: 'set to `false`, it deactivates the pagination (returns all objects)'
          HOUSING_APPLICATIONS_ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = :string
            if HOUSING_APPLICATIONS_MUTATE_PARAMS[acceptable_param]
              type = nice_type(MUTATE_PARAMS[acceptable_param])
            end
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end
          # VALID_OPERATORS.each do |valid_operator|
          #   parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
          # end

          security [Bearer: {}]
          paginator

          response '200', 'housing_offers found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:id) {housing_offer.id.to_s}
            let(:from) {(Time.now - 2.hours).to_s}
            let(:to) {(Time.now + 1.hours).to_s}
            let(:pagination) {'false'}

            include_context 'with integration test'
            include_context 'valid jsonapi collection without pagination' do
              let(:expected_collection) {housing_applications}
            end
          end
        end
      end

      path '/api/dmo/housing_offers/{id}/selected_applications' do
        get 'Retrieves housing_applications relative to a given housing_offer where the status is ...' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          parameter name: 'ids[]', in: :query, type: :array, items: {type: :string}, collectionFormat: :multi, required: false
          security [Bearer: {}]

          HOUSING_APPLICATIONS_ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = :string
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end

          paginator
          response '200', 'housing_application found' do
            let!(:housing_offer1) {create(:housing_offer)}
            let(:'Content-Type') {'multipart/form-data'}
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:id) {housing_offer1.id}
            let!(:housing_applications) do
              create_list(:housing_application, 5, housing_offer_id: housing_offer1.id.to_s,
                          housing_request_id: housing_request.id.to_s,
                          status: 'PLO')
            end
            let!(:housing_applications2) do
              create_list(:housing_application, 5, housing_offer_id: housing_offer1.id.to_s,
                          housing_request_id: housing_request.id.to_s,
                          status: 'TRANSMIS BAILLEUR')
            end
            run_test!
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'
            after do |_example|
              json_data = JSON.parse(response.body)['data']
              expect(json_data.length).to eq(5)
            end
          end
        end
      end

      path '/api/dmo/housing_offers/{id}/detailed_score' do
        get 'Retrieves the detailed score relative to a given housing_offer for one applicant' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          parameter(
            name: 'applicant_id',
            in: :query,
            type: :string,
            required: true,
            description: '<span>' \
              '<div class="opblock opblock-get" style="padding: 4px; align-items: center; display: flex">' \
              '<span class="opblock-summary-method" style="margin-right: 4px">NOTICE</span>' \
              '<span class="no-style">The `applicant_id` param is required ' \
              ' unless the current role is "Demandeur".</span></div></span>'
          )
          security [Bearer: {}]

          paginator
          response '200', 'score found' do
            let!(:housing_offer) {create(:housing_offer)}
            let!(:applicant) {create :applicant}
            let!(:score) {create :score, housing_offer_id: housing_offer.id, applicant_id: applicant.id}
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:id) {housing_offer.id}
            let(:applicant_id) {applicant.id}

            run_test!
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'
            after do |_example|
              json_data = JSON.parse(response.body)['data']
              expect(json_data.length).to eq 1
            end
          end
        end
      end

      path '/api/dmo/housing_offers/{id}/selected_applications' do
        post 'set the selected requests for an housing offer' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          json_params = {}
          json_params[:als_request_ids] = {type: :array, items: {type: :string}, collectionFormat: :multi, required: true}
          parameter(
            name: :params,
            in: :body,
            schema: jsonify_params(json_params),
            description: '<span>' \
              '<div class="opblock opblock-delete" style="padding: 4px; align-items: center; display: flex">' \
              '<span class="opblock-summary-method" style="margin-right: 4px">WARNING</span>' \
              '<span class="no-style">The only param is "request_als_ids", and it correspond' \
              ' to the id of the "demandes" as registered in als back</span></div></span>'
          )

          security [Bearer: {}]
          response '200', 'housing_applications updated' do
            let!(:housing_offer1) {create(:housing_offer)}

            let!(:user7) {create(:user)}
            let!(:applicant7) {create(:applicant, user: user7)}
            let!(:housing_request7) {create(:housing_request, main_applicant_id: applicant7.id.to_s, als_id: 'als_id7')}
            let!(:housing_applications7) do
              create(:housing_application,
                     housing_offer_id: housing_offer1.id.to_s,
                     housing_request_id: housing_request7.id.to_s,
                     status: 'PLO')
            end
            let!(:user8) {create(:user)}
            let!(:applicant8) {create(:applicant, user: user8)}
            let!(:housing_request8) {create(:housing_request, main_applicant_id: applicant8.id.to_s, als_id: 'als_id8')}
            let!(:housing_applications8) do
              create(:housing_application,
                     housing_offer_id: housing_offer1.id.to_s,
                     housing_request_id: housing_request8.id.to_s,
                     status: 'PLO')
            end

            let(:'Content-Type') {'multipart/form-data'}
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:id) {housing_offer1.id}
            let(:body) do
              {data: {
                attributes: {
                  als_request_ids: [housing_request7.als_id, housing_request8.als_id]
                }

              }}
            end

            run_test!

            after do |_example|
            end
          end
        end
      end

      # Pictures functionalities
      path '/api/dmo/housing_offers/{housing_offer_id}/pictures' do
        post 'create_picture for an housing_offer' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          parameter name: 'Content-Type', in: :header, type: :string, required: false, description: "must be 'multipart/form-data'"
          consumes 'multipart/form-data'
          # consumes 'application/json'
          parameter name: :housing_offer_id, in: :path, type: :string
          security [Bearer: {}]
          parameter name: :file, in: :formData, type: :file, description: 'picture to upload'
          description 'Send a file with the post action to add a picture to the housing_offer. you can only add 5 pictures, use delete and then post do add others or use put to change one.<br> Edit the housing_offer[ordered_picture_ids] field. the main picture is the first of the array. <br> The main picture is the first of the array'

          response '201', 'picture created' do
            let(:file) do
              Rack::Test::UploadedFile.new([Hermes::Engine.root, %w[lib hermes tools minion.jpg]].flatten.join('/'), 'mime/type')
            end
            let!(:housing_offer1) {create(:housing_offer)}
            let(:'Content-Type') {'multipart/form-data'}
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:housing_offer_id) {housing_offer1.id}
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json_data = JSON.parse(response.body)['data']
              ho = Hermes::HousingOffer.find_by(id: housing_offer1.id)
              expect(json_data).to be_a Hash
              expect(json_data['id']).to_not be nil
              expect(json_data['type']).to eq 'pictures'
              expect(json_data['attributes']['housing_offer_id']).to eq(housing_offer1.id.to_s)
              expect(json_data['attributes']['thumb240']).to_not be_nil
              expect(
                Hermes::HousingOfferSerializer.new(
                  Hermes::HousingOffer.find_by(id: json_data['attributes']['housing_offer_id']),
                  scope: responsible,
                  root: true
                ).attributes[:pictures_nb]
              ).to eq(1)
              expect(ho.ordered_picture_ids).to eq([json_data['id'].to_s])
            end
          end
        end
      end
      path '/api/dmo/housing_offers/{housing_offer_id}/pictures/{picture_id}' do
        put 'edit_picture for an housing_offer' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          parameter name: :housing_offer_id, in: :path, type: :string
          parameter name: :picture_id, in: :path, type: :string
          produces 'application/json'
          parameter name: 'Content-Type', in: :header, type: :string, required: false, description: "must be 'multipart/form-data'"
          consumes 'multipart/form-data'
          parameter name: :file, in: :formData, type: :file, description: 'picture to upload'
          security [Bearer: {}]
          description 'Send a file with the update action to edit one of the picture. if you want to reorder picture, edit the housing_offer[ordered_picture_ids] field. the main picture is the first of the array'

          response '200', 'picture edited' do
            let(:file) do
              Rack::Test::UploadedFile.new([Hermes::Engine.root, %w[lib hermes tools minion.jpg]].flatten.join('/'), 'mime/type')
            end
            let!(:housing_offer1) {create(:housing_offer)}
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:housing_offer_id) {housing_offer1.id}
            let(:picture1) {create(:picture, housing_offer_id: housing_offer1.id, user_id: responsible.id.to_s)}
            let(:picture_id) {picture1.id.to_s}
            let(:old_assets_ids) {[picture1.thumb240_id, picture1.thumb1000_id, picture1.full_size_id]}
            let(:'Content-Type') {'multipart/form-data'}
            # include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'
            after do |_example|
              json_data = JSON.parse(response.body)['data']
              expect(json_data).to be_a Hash
              expect(json_data['id']).to_not be nil
              edited_pic = Hermes::Picture.find_by(id: picture1.id)
              expect(json_data['type']).to eq 'pictures'
              expect(edited_pic.thumb240_id).not_to eq(old_assets_ids[0])
              expect(edited_pic.thumb1000_id).not_to eq(old_assets_ids[1])
              expect(edited_pic.full_size_id).not_to eq(old_assets_ids[2])
            end
          end
        end
      end
      path '/api/dmo/housing_offers/{housing_offer_id}/pictures/{picture_id}' do
        delete 'delete_picture' do
          let!(:housing_offer1) {create(:housing_offer)}
          let(:Authorization) {"Bearer #{token_for(responsible)}"}
          let(:housing_offer_id) {housing_offer1.id}
          let(:picture1) {create(:picture, user_id: responsible.id, housing_offer_id: housing_offer1.id)}

          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :housing_offer_id, in: :path, type: :string
          parameter name: :picture_id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          json_params[:description] = {type: :string}
          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '204', 'picture deleted' do
            let(:picture_id) {picture1.id.to_s}
            let(:params) do
              jsonapi_attributes(
                {
                }
              )
            end
            run_test!
            after do |_example|
              ho = Hermes::HousingOffer.find_by(id: housing_offer1.id)
              expect(ho.ordered_picture_ids).to eq([])
            end
          end
        end
      end

      path '/api/dmo/housing_offers/{id}' do
        put 'edit housing_offer' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          json_params[:lessor_id] = {type: :string}
          json_params[:construction_year] = {type: :number}
          parameter(
            name: :params,
            in: :body,
            schema: jsonify_params(json_params),
            description: '<span>' \
              '<div class="opblock opblock-delete" style="padding: 4px; align-items: center; display: flex">' \
              '<span class="opblock-summary-method" style="margin-right: 4px">WARNING</span>' \
              '<span class="no-style">Setting an array of siret within `entities_siret`, will ' \
              ' change its relationships with the corresponding entities.</span></div></span>'
          )
          # jsonapi_params( **json_params )

          response '200', 'housing_offers updated' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:housing_offer) {create(:housing_offer)}
            let(:id) {housing_offer.id}
            let(:params) do
              jsonapi_attributes(
                external_ref: 'String'
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'
            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_offers'
            end
          end
        end
      end

      path '/api/dmo/housing_offers/{id}' do
        delete 'Delete a single housing_offer by id' do
          tags TAGS_API_DMO_HOUSING_OFFERS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '204', 'housing_offer deleted' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:id) {create(:housing_offer, user_id: responsible.id).id}
            run_test!
          end
        end
      end
    end
  end
end
